#include <math.h>
#include <qpainter.h>

#ifdef QT4
#include <classcompathack.h>
#endif

#include "coverartflow.h"
#include "debug.h"
#include "mediadatabase.h"
#include "sqlite3.h"

#include "compathack.h"

/* CoverArtFlow */

CoverArtFlow::CoverArtFlow(ReflectionImageProvider *imageProvider, LayeredPaintBox *parent)
	: 	ReflectionImageFlow(imageProvider, parent),
		mediaDatabase_(NULL),
		aspectSource_(NULL),
		playListView_(NULL)
{
	setTransitionTime(500);
	setVerticalOffset(ReflectionHeight / 4);
	//setConstantVerticalOffset(-80);
	setReflectionHeight(ReflectionHeight);
	setSlideSize(QSize(SlideWidthHeight, SlideWidthHeight + ReflectionHeight));

	connect(&preloadImagesTimer_, SIGNAL(timeout()), this, SLOT(preloadImages()));
	connect(&focusAlbumTimer_, SIGNAL(timeout()), this, SLOT(focusCurrentAlbum()));
}

CoverArtFlow::~CoverArtFlow()
{
}

void CoverArtFlow::attachTo(PlayListAspectAlbumGroupBuilder *source)
{
	detach();

	if (source)
	{
		aspectSource_ = source;
		mediaDatabase_ = aspectSource_->playListAspect()->playList()->mediaDatabase();

		updateView();

		if (aspectSource_->selectedIndex() > 0)
			jumpTo(aspectSource_->selectedIndex());

		connect(aspectSource_, SIGNAL(updateView()), this, SLOT(updateView()));
	}
}

void CoverArtFlow::detach()
{
	if (aspectSource_)
	{
		disconnect(aspectSource_, SIGNAL(updateView()), this, SLOT(updateView()));

		aspectSource_->setSelectedIndex((int)floor(currentValue()));

		reset();

		mediaDatabase_ = NULL;
		aspectSource_ = NULL;
	}
}

void CoverArtFlow::setPlayListView(PlayListView *playList)
{
	if (playListView())
		disconnect(playListView(), SIGNAL(currentChanged(QListViewItem *)), this, SLOT(playListCurrentChanged(QListViewItem *)));

	playListView_ = playList;

	if (playListView())
		connect(playListView(), SIGNAL(currentChanged(QListViewItem *)), this, SLOT(playListCurrentChanged(QListViewItem *)));
}

/*
void CoverArtFlow::extensionGraphExecutionFinished()
{
	if (aspectSource_ && playListView_ && itemCount() > 0)
	{
		if (playListView()->currentItem())
		{
			PlayListViewItem *item = static_cast<PlayListViewItem *>(playListView()->currentItem());
			jumpTo(aspectSource_->groupIndexForPlayListID(item->id()));
		}
		else
			jumpTo(0);
	}
}
*/

void CoverArtFlow::playListCurrentChanged(QListViewItem *lvi)
{
	if (aspectSource_ && playListView_ && itemCount() > 0)
	{
		if (lvi)
		{
			PlayListViewItem *item = static_cast<PlayListViewItem *>(lvi);
			transitionTo(aspectSource_->groupIndexForPlayListID(item->id()));
		}
		else
			transitionTo(0);
	}
}

void CoverArtFlow::transitionFinished()
{
	DENTERMETHOD;

	CustomPerspectiveImageFlow::transitionFinished();
	focusAlbumTimer_.start(500, true);

	DEXITMETHOD;
}

void CoverArtFlow::focusCurrentAlbum()
{
	if (!aspectSource_ || !playListView_) return;

	int itemIndex = floor(currentValue());
	int itemIndexViaPlaylist = -1;

	// Try to get the item index via the currently selected playlist item...
	if (playListView()->currentItem())
		itemIndexViaPlaylist = aspectSource_->groupIndexForPlayListID(static_cast<PlayListViewItem *>(playListView()->currentItem())->id());

	// Only select the first song if the particular group isn't selected yet
	if (itemIndex != itemIndexViaPlaylist)
	{
		// Make sure the album is visible in the list
		// This is a workaround to avoid having the first song
		// of this particular group as last item in the visible area...
		if (itemIndex < aspectSource_->groups().count() - 1)
		{
			PlayListViewItem *playlistItem = playListView()->findItemByID(aspectSource_->groups().at(itemIndex + 1)->playlistID);
			playListView()->ensureItemVisible(playlistItem);
		}
		else
			playListView()->ensureItemVisible(playListView()->lastItem());

		PlayListViewItem *playlistItem = playListView()->findItemByID(aspectSource_->groups().at(itemIndex)->playlistID);
		playListView()->ensureItemVisible(playlistItem);
		playListView()->setCurrentItem(playlistItem);
	}
}

void CoverArtFlow::mouseDoubleClickEvent(QMouseEvent *e)
{
	if (!playListView_) return;

	focusCurrentAlbum();
	playListView()->triggerPlaybackIntent();
}

void CoverArtFlow::paintOverlay(QPainter &p, const QPixmap &pixmap)
{
	ReflectionImageFlow::paintOverlay(p, pixmap);

	if (!aspectSource_) return;

	p.setPen(QColor(255, 255, 255));
	p.setBrush(Qt::NoBrush);

	int itemIndex = floor(currentValue());

	if (itemIndex >= 0 && itemIndex < aspectSource_->groups().count())
	{
		GroupItem *item = aspectSource_->groups().at(itemIndex);

		//int boxHeight = floor((reflectionHeight() - verticalOffset()) * scale());
		int boxHeight = 40;
#ifdef QTOPIA
		int fontHeight = 15; // TODO: FIXME!
#else
		int fontHeight = p.fontInfo().pixelSize();
#endif

		QRect rect;

		if (boxHeight < fontHeight * 2)
		{
			rect = QRect(0, height() - fontHeight - 2 - 40, width(), fontHeight + 2);
			p.drawText(rect, Qt::AlignCenter, item->artist + " - " + item->album);
		}
		else
		{
			rect = QRect(0, height() - boxHeight - 40, width(), boxHeight);
			p.drawText(rect, Qt::AlignCenter, item->artist + "\n" + item->album);
		}
	}

	if (imageProvider() && imageProvider()->isWorking())
	{
		p.drawText(5, 25, "Loading...");
	}
}

/*
void CoverArtFlow::setActive(bool value)
{
	active_ = value;

	if (active_)
		updateView();
	else
	{
		imageProvider()->flushMemoryCache();
	}
}
*/

void CoverArtFlow::updateView()
{
	if (!aspectSource_) return;

	DENTERMETHOD;

	setItemCount(aspectSource_->groups().count());
	preloadImages();

	DEXITMETHOD;
}

QImage CoverArtFlow::getImage(int index)
{
	if (aspectSource_ && index >= 0 && index < aspectSource_->groups().count())
	{
		GroupItem *item = aspectSource_->groups().at(index);
		if (item)
		{
			QImage *image = imageProvider()->getImage(item->key, item);

			if (image != NULL)
				return *image;
			else
				return QImage();
		}
		else
			return QImage();
	}
	else
		return QImage();
}

void CoverArtFlow::preloadImages()
{
	if (!aspectSource_) return;

	imageProvider()->lock();

	QStringList keys;

	GroupItem *item;

	for (int i = 0; i < aspectSource_->groups().count(); ++i)
	{
		item = aspectSource_->groups().at(i);

		if (!imageProvider()->imageExists(item->key))
			imageProvider()->queue(item->key, item);

		keys.append(item->key);
	}

	imageProvider()->prefetchHint(keys);

	imageProvider()->unlock();
}

void CoverArtFlow::imageWasCached(CachedImage *image)
{
	ReflectionImageFlow::imageWasCached(image);

	if (!preloadImagesTimer_.isActive())
		preloadImagesTimer_.start(1000, true);
}

void CoverArtFlow::reset()
{
	imageProvider()->resetNullImageNameList();
	imageProvider()->flushMemoryCache();
}
